//+------------------------------------------------------------------+
//|                                                 ATR Channels.mq4 |
//|                         Copyright  2005, Luis Guilherme Damiani |
//|                                      http://www.damianifx.com.br |
//+------------------------------------------------------------------+

#property copyright "Copyright  2005, Luis Guilherme Damiani"
#property link      "http://www.damianifx.com.br"
#property indicator_chart_window
#property  indicator_buffers 7
#property  indicator_color1  clrGreen       //Moving Average
#property  indicator_color2  clrDeepSkyBlue // Lower band 1
#property  indicator_color3  clrDeepSkyBlue // Upper band 1
#property  indicator_color4  clrBlue        // Lower band 2
#property  indicator_color5  clrBlue        // Upper band 2
#property  indicator_color6  clrRed         // Lower band 3
#property  indicator_color7  clrRed         // Upper band 3


extern ENUM_TIMEFRAMES TimeFrame    = PERIOD_CURRENT;
extern int             PeriodsATR                 = 18;
extern int             MA_Periods                 = 49;
extern ENUM_MA_METHOD  MA_type                    = MODE_LWMA;
extern double          Mult_Factor1               = 1.6;
extern double          Mult_Factor2               = 3.2;
extern double          Mult_Factor3               = 4.8;
extern bool            AlertsandArrowsOnHighLow   = true;
extern bool            alertsOn                   = true;
extern bool            alertsOnLevel1Cross        = true;
extern bool            alertsOnLevel2Cross        = true;
extern bool            alertsOnLevel3Cross        = true;
extern bool            alertsOnCurrent            = false;
extern bool            alertsMessage              = true;
extern bool            alertsSound                = true;
extern bool            alertsNotify               = false;
extern bool            alertsEmail                = false;
extern string          soundFile                  = "alert2.wav"; 
extern bool            arrowsVisible              = true;
extern string          arrowsIdentifier           = "ATRCH Arrows1";
extern double          arrowsUpperGap             = 0.5;
extern double          arrowsLowerGap             = 0.5;
extern bool            arrowsOnLevel1Cross        = false;
extern color           arrowsOnLevel1CrossUpColor = clrDeepSkyBlue;
extern color           arrowsOnLevel1CrossDnColor = clrRed;
extern int             arrowsOnLevel1CrossUpCode  = 233;
extern int             arrowsOnLevel1CrossDnCode  = 234;
extern int             arrowsOnLevel1CrossUpSize  = 1;
extern int             arrowsOnLevel1CrossDnSize  = 1;
extern bool            arrowsOnLevel2Cross        = false;
extern color           arrowsOnLevel2CrossUpColor = clrDeepSkyBlue;
extern color           arrowsOnLevel2CrossDnColor = clrRed;
extern int             arrowsOnLevel2CrossUpCode  = 233;
extern int             arrowsOnLevel2CrossDnCode  = 234;
extern int             arrowsOnLevel2CrossUpSize  = 1;
extern int             arrowsOnLevel2CrossDnSize  = 1;
extern bool            arrowsOnLevel3Cross        = false;
extern color           arrowsOnLevel3CrossUpColor = clrDeepSkyBlue;
extern color           arrowsOnLevel3CrossDnColor = clrRed;
extern int             arrowsOnLevel3CrossUpCode  = 233;
extern int             arrowsOnLevel3CrossDnCode  = 234;
extern int             arrowsOnLevel3CrossUpSize  = 1;
extern int             arrowsOnLevel3CrossDnSize  = 1;

double ma[];
double Ch1up[];
double Ch1dn[];
double Ch2up[];
double Ch2dn[];
double Ch3up[];
double Ch3dn[];
double trend1[];
double trend2[];
double trend3[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//---- input parameters

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(10);
   SetIndexBuffer(0, ma);
   SetIndexBuffer(1, Ch1up);
   SetIndexBuffer(2, Ch1dn);
   SetIndexBuffer(3, Ch2up);
   SetIndexBuffer(4, Ch2dn);
   SetIndexBuffer(5, Ch3up);
   SetIndexBuffer(6, Ch3dn);
   SetIndexBuffer(7, trend1);
   SetIndexBuffer(8, trend2);
   SetIndexBuffer(9, trend3);
   
      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
      
      //
      //
      //
      //
      //
      
      IndicatorShortName(timeFrameToString(TimeFrame)+" ATR-Channels");
return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
      if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
return (0);
}
   
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ma[0] = limit+1; return(0); }
           
   //
   //
   //
   //
   //
           
   if (TimeFrame == Period())
   {
     for (i=limit; i>=0; i--)
     {
        double atr = iATR(NULL,0,PeriodsATR,i);
               ma[i]    = iMA(NULL,0,MA_Periods,0,MA_type,PRICE_TYPICAL,i);
               Ch1up[i] = ma[i] + atr*Mult_Factor1;
               Ch1dn[i] = ma[i] - atr*Mult_Factor1;
               Ch2up[i] = ma[i] + atr*Mult_Factor2;
               Ch2dn[i] = ma[i] - atr*Mult_Factor2;
               Ch3up[i] = ma[i] + atr*Mult_Factor3;
               Ch3dn[i] = ma[i] - atr*Mult_Factor3;
               
               trend1[i] = 0;
               trend2[i] = 0;
               trend3[i] = 0;
               if (AlertsandArrowsOnHighLow)       
               {
                 if (Low[i] <Ch1dn[i])   trend1[i] = 1;
                 if (High[i]>Ch1up[i])   trend1[i] =-1;
                 if (Low[i] <Ch2dn[i])   trend2[i] = 1;
                 if (High[i]>Ch2up[i])   trend2[i] =-1;
                 if (Low[i] <Ch3dn[i])   trend3[i] = 1;
                 if (High[i]>Ch3up[i])   trend3[i] =-1;
               }
               else
               {
                 if (Close[i]<Ch1dn[i])  trend1[i] = 1;
                 if (Close[i]>Ch1up[i])  trend1[i] =-1;
                 if (Close[i]<Ch2dn[i])  trend2[i] = 1;
                 if (Close[i]>Ch2up[i])  trend2[i] =-1;
                 if (Close[i]<Ch3dn[i])  trend3[i] = 1;
                 if (Close[i]>Ch3up[i])  trend3[i] =-1;
               }
         
      
         //
         //
         //
         //
         //
               
         if (arrowsVisible)
         {
            ObjectDelete(arrowsIdentifier+":1:"+Time[i]);
            ObjectDelete(arrowsIdentifier+":2:"+Time[i]);
            ObjectDelete(arrowsIdentifier+":3:"+Time[i]);
            string lookFor = arrowsIdentifier+":"+Time[i]; ObjectDelete(lookFor);
            if (arrowsOnLevel1Cross && trend1[i] != trend1[i+1])
            {
               if (trend1[i] == 1) drawArrow("1",0.5,i,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossUpSize,false);
               if (trend1[i] ==-1) drawArrow("1",0.5,i,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossDnSize,true);
            } 
            if (arrowsOnLevel2Cross && trend2[i] != trend2[i+1]) 
            { 
               if (trend2[i] == 1) drawArrow("2",1.0,i,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossUpSize,false);
               if (trend2[i] ==-1) drawArrow("2",1.0,i,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossDnSize,true);                                              
            }  
            if (arrowsOnLevel3Cross && trend3[i] != trend3[i+1]) 
            { 
               if (trend3[i] == 1) drawArrow("3",1.5,i,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossUpSize,false);
               if (trend3[i] ==-1) drawArrow("3",1.5,i,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossDnSize,true);                               
     
            }
          } 
       }
       
       //
       //
       //
       //
       //
       
       if (alertsOn)
       {
         if (alertsOnCurrent)
              int whichBar = 0;
         else     whichBar = 1; 
      
         if (alertsOnLevel1Cross && trend1[whichBar] != trend1[whichBar+1])
         {
           static string   mess1 = "";
           static datetime time1 = 0;        
           if (trend1[whichBar] ==  1) doAlert(mess1,time1,whichBar,"price crossing first lower level");
           if (trend1[whichBar] == -1) doAlert(mess1,time1,whichBar,"price crossing first upper level");
         } 
         if (alertsOnLevel2Cross && trend2[whichBar] != trend2[whichBar+1])
         {
            static string   mess2 = "";
            static datetime time2 = 0;        
            if (trend2[whichBar] == 1)  doAlert(mess2,time2,whichBar,"price crossing second lower level");
            if (trend2[whichBar] ==-1)  doAlert(mess2,time2,whichBar,"price crossing second upper level");
         
         } 
         if (alertsOnLevel3Cross && trend3[whichBar] != trend3[whichBar+1])
         {
            static string   mess3 = "";
            static datetime time3 = 0;        
            if (trend3[whichBar] == 1)  doAlert(mess3,time3,whichBar,"price crossing third lower level");
            if (trend3[whichBar] ==-1)  doAlert(mess3,time3,whichBar,"price crossing third upper level");
         
         }                            
      }
      return(0);
      }
      
      //
      //
      //
      //
      //
      
      limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
      for (i=limit;i>=0; i--)
      {
         int y = iBarShift(NULL,TimeFrame,Time[i]);
            ma[i]    = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,0,y);
            Ch1up[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,1,y);
            Ch1dn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,2,y);
            Ch2up[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,3,y);
            Ch2dn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,4,y);
            Ch3up[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,5,y);
            Ch3dn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,PeriodsATR,MA_Periods,MA_type,Mult_Factor1,Mult_Factor2,Mult_Factor3,AlertsandArrowsOnHighLow,alertsOn,alertsOnLevel1Cross,alertsOnLevel2Cross,alertsOnLevel3Cross,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,
                       arrowsOnLevel1Cross,arrowsOnLevel1CrossUpColor,arrowsOnLevel1CrossDnColor,arrowsOnLevel1CrossUpCode,arrowsOnLevel1CrossDnCode,arrowsOnLevel1CrossUpSize,arrowsOnLevel1CrossDnSize,
                       arrowsOnLevel2Cross,arrowsOnLevel2CrossUpColor,arrowsOnLevel2CrossDnColor,arrowsOnLevel2CrossUpCode,arrowsOnLevel2CrossDnCode,arrowsOnLevel2CrossUpSize,arrowsOnLevel2CrossDnSize,
                       arrowsOnLevel3Cross,arrowsOnLevel3CrossUpColor,arrowsOnLevel3CrossDnColor,arrowsOnLevel3CrossUpCode,arrowsOnLevel3CrossDnCode,arrowsOnLevel3CrossUpSize,arrowsOnLevel3CrossDnSize,6,y);  
       }
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
  
//
//
//
//
//

void doAlert(string& previousAlert, datetime& previousTime, int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Atr Channels ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Atr Channels "),message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void drawArrow(string nameAdd, double gapMul, int i,color theColor,int theCode,int theSize, bool up)
{
   string tname = arrowsIdentifier+":"+nameAdd+":"+Time[i];
   double gap   = iATR(NULL,0,20,i)*gapMul;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(tname,OBJ_ARROW,0,Time[i],0);
         ObjectSet(tname,OBJPROP_ARROWCODE,theCode);
         ObjectSet(tname,OBJPROP_COLOR,theColor);
         ObjectSet(tname,OBJPROP_WIDTH,theSize );  
         if (up)
               ObjectSet(tname,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(tname,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

